"""
Functionality: This is the main script to generate a matrix of features of interest.

Arguments:
1) intermediate_participant_path
e.g., G:\...\intermediate_file\sharpnessnextpouch@timestudy_com

2) output_dir_path
e.g., C:\...\output_folder

3) date_in_study
e.g., 2021-01-01

4) decryption_password
eg., passwordpassword

Command line example:
python main.py [intermediate_participant_path] [output_dir_path] [date_in_study] [decryption_password]

Output: feature_mx_<date>.csv file that contains all features in the <output_dir_path> folder .
"""
import sys
import time
import os
import pandas as pd
from microt_compliance_matrix import utils
from microt_compliance_matrix import feature_engineering_ema


def compute_features_phone(intermediate_participant_path, output_dir_path, date_in_study, lsf_json_folder_path,
                           decryption_password):
    # Switch
    print_stats = False

    # Extract participant id from participant folder path
    participant_id = utils.extract_participant_id.extract_participant_id(intermediate_participant_path)

    # check if skip
    csv_file_name = "ema_feature_mx_{}".format(date_in_study) + ".csv"
    csv_save_path = os.path.join(output_dir_path, participant_id, csv_file_name)
    if os.path.exists(csv_save_path):
        print("Output exists. Skip EMA for {}-{}".format(participant_id, date_in_study))
        return

    # Create output folder
    output_participant_path = os.path.join(output_dir_path, participant_id)
    if not os.path.exists(output_participant_path):
        try:
            os.makedirs(output_participant_path)
        except FileExistsError:
            pass

    # Parse prompt response file for the specific date
    df_prompt = feature_engineering_ema.prompt_response.preprocess_promptResponse_raw.clean_response_files(
        intermediate_participant_path, date_in_study, participant_id)

    # - Outcome variable: completion status
    completion_status_binary = feature_engineering_ema.compliance.create_column(df_prompt['Answer_Status'],
                                                                                print_stats)
    # - Feature matrix
    feature_df = df_prompt
    feature_df['Answer_Status'] = completion_status_binary

    # - Loading Participants Information
    info_dict = utils.load_participant_study_info.execute(intermediate_participant_path, output_dir_path, lsf_json_folder_path)

    # - feature 1: Day of the week
    feature1 = feature_engineering_ema.day_of_the_week.create_column(df_prompt['Actual_Prompt_Local_DateTime'], print_stats)
    feature_df['DAY_OF_THE_WEEK'] = feature1
    # - feature 2: Time of the day
    feature2 = feature_engineering_ema.time_of_the_day.create_column(df_prompt['Actual_Prompt_Local_DateTime'], print_stats)
    feature_df['TIME_OF_THE_DAY'] = feature2
    # - feature 3: Days in the study
    feature3 = feature_engineering_ema.days_in_study.create_column(df_prompt, info_dict, print_stats)
    feature_df['DAYS_IN_THE_STUDY'] = feature3

    # - feature 4: battery level
    feature5, feature6, match_time_battery = feature_engineering_ema.battery_level.create_column(
        df_prompt['Actual_Prompt_Local_DateTime'], intermediate_participant_path, date_in_study)
    feature_df['BATTERY_LEVEL'] = feature5
    feature_df['CHARGING_STATUS'] = feature6
    feature_df['match_time_battery'] = match_time_battery

    # - feature 5: GPS location
    feature7, match_time_location, feature7_1 = feature_engineering_ema.location.create_column(participant_id, df_prompt['Actual_Prompt_Local_DateTime'], info_dict,
                                                                                   intermediate_participant_path,
                                                                                   date_in_study, decryption_password)
    feature_df['LOCATION'] = feature7
    feature_df['match_time_location'] = match_time_location
    feature_df['DISTANCE_FROM_HOME'] = feature7_1

    # - feature 6: Location label
    feature8 = feature_engineering_ema.location_label.create_column(lsf_json_folder_path, participant_id, feature7)
    feature_df['LOCATION_LABEL'] = feature8

    # - feature 7: Screen Status
    feature9, match_time_screen = feature_engineering_ema.screen_status.create_column(
        df_prompt['Actual_Prompt_Local_DateTime'], intermediate_participant_path, date_in_study)
    feature_df['SCREEN_STATUS'] = feature9
    feature_df['match_time_screen'] = match_time_screen

    # - feature 10&11: wake and sleep hour
    feature10, feature11 = feature_engineering_ema.waking_hours.create_column(df_prompt['Actual_Prompt_Local_DateTime'],
                                                                              intermediate_participant_path,
                                                                              date_in_study)
    feature_df['WAKE_TIME'] = feature10
    feature_df['SLEEP_TIME'] = feature11

    # - feature 12: Phone lock status
    feature12, feature13, match_time_lock = feature_engineering_ema.phone_lock.create_column(
        df_prompt['Actual_Prompt_Local_DateTime'], intermediate_participant_path, date_in_study)
    feature_df['PHONE_LOCK'] = feature12
    feature_df['LAST_USAGE_DURATION'] = feature13
    feature_df['match_time_lock'] = match_time_lock

    # - feature 13: parts of waking hour
    parts_num = 4
    feature14, feature15, feature16 = feature_engineering_ema.parts_of_waking_hour.create_column(feature_df, parts_num)
    feature_df['PARTS_OF_WAKING_HOUR'] = feature14
    feature_df['PROXIMITY_TO_WAKE_TIME'] = feature15
    feature_df['PROXIMITY_TO_SLEEP_TIME'] = feature16

    # - feature 14: MIMS Summary
    df_mims = feature_engineering_ema.mims_summary.create_column(df_prompt['Actual_Prompt_Local_DateTime'],
                                                                 intermediate_participant_path, date_in_study)
    feature_df = pd.concat([feature_df, df_mims], axis=1)


    # save
    feature_df.to_csv(csv_save_path, index=False)

    return


if __name__ == "__main__":
    start = time.time()
    intermediate_participant_path = sys.argv[1]
    output_dir_path = sys.argv[2]
    date_in_study = sys.argv[3]
    lsf_json_folder_path = sys.argv[4]
    decryption_password = sys.argv[5]

    compute_features_phone(intermediate_participant_path, output_dir_path, date_in_study, lsf_json_folder_path,
                           decryption_password)
    end = time.time()
    print("Finished\n")
    print("Runtime : {} seconds".format(round(end - start, 2)))



from math import radians, cos, sin, asin, sqrt

def haversine(loc1, loc2):
    """
    Calculate the great circle distance in kilometers between two points
    on the earth (specified in decimal degrees)
    """
    # convert decimal degrees to radians
    lon1 = loc1[1]
    lat1 = loc1[0]
    lon2 = loc2[1]
    lat2 = loc2[0]
    if lat1 > 7:  # already radians
        lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])

    # haversine formula
    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = sin(dlat/2)**2 + cos(lat1) * cos(lat2) * sin(dlon/2)**2
    c = 2 * asin(sqrt(a))
    r = 6371 # Radius of earth in kilometers. Use 3956 for miles. Determines return value units.
    return c * r

if __name__ == "__main__":
    lon1 = -103.548851
    lat1 = 32.0004311
    lon2 = -103.6041946
    lat2 = 33.374939

    print(haversine([lat1, lon1], [lat2, lon2]))


from os import listdir
import re
from datetime import datetime
import os


def verifyDateFolder(dates_list):
    existed_folder_list = [x for x in dates_list if bool(re.match(r'(\d+\-\d+\-\d+)', x))]
    return existed_folder_list


def filterDates(dates_list, date_start, date_end):
    dates_to_check = []
    date_format = '%Y-%m-%d'

    date_start_entered = datetime.strptime(date_start, date_format)
    date_end_entered = datetime.strptime(date_end, date_format)

    dt_dates = [datetime.strptime(date, date_format) for date in dates_list]
    dt_dates_sorted = sorted(dt_dates)

    for date in dt_dates_sorted:
        if (date >= date_start_entered) and (date <= date_end_entered):
            dates_to_check.append(date.strftime(date_format))

    return dates_to_check


def validate_date(date_folder_path):
    date_exist = os.path.exists(date_folder_path)

    return date_exist


if __name__ == "__main__":
    area_folder_path = r"E:\data\wocket\Wockets-win32-x64\resources\app\src\srv\MICROT\aditya4_internal@timestudy_com\logs-watch"
    date_start = "2020-01-01"
    date_end = "2020-07-01"

    validated_date_list = validate_date(area_folder_path, date_start, date_end)
    print("Total number of valid date folders  :  {}".format(len(validated_date_list)))
    print(validated_date_list)



"""
Output: 1) a list of targeted features for the participant 2) print stats of the targeted features for the participant
"""

from os import path, sep
import pandas as pd
from datetime import datetime
from collections import Counter
import sys


def print_stats_single_column(feature_name, data):
    print('--------- Stats of Feature: ' + feature_name + ' ---------')
    counter = Counter(data)
    print(sorted(counter.items()))


def categorize_timeOfDay(dateobject):
    hour = dateobject.hour
    # TBD
    # if 4 < hour and hour < 12:
    #     return "morning"
    # elif hour > 12 and hour < 17:
    #     return "afternoon"
    # else:
    #     return "evening/night"
    return hour

def create_column(prompt_local_datetime_series, print_stats=True):
    print("\n> start generating the feature: time of the day")

    # map from date object to weekday
    prompt_timeOfDay_column = list(map(lambda x: categorize_timeOfDay(x), prompt_local_datetime_series))

    if print_stats == True:
        print_stats_single_column("timeOfDay", prompt_timeOfDay_column)

    return prompt_timeOfDay_column

